ad_page_contract {
    Administration page for intranet-search-oracle

    @param project_id   The project to be trained
    @author Marc Baiges Camprub (tm08707@salleurl.edu)
} {    
    project_id:notnull
    tema:array,notnull
}

# ---------------------------------------------------------------------
# Defaults & Security
# ---------------------------------------------------------------------

set user_id [ad_maybe_redirect_for_registration]

# Initialize varialbes
global contingut
set contingut ""
global count_list
set count_list [list]

# ---------------------------------------------------------------------
# Proc to read a file's content
# ---------------------------------------------------------------------
proc read_file { name } {
    set f [open $name]
         set data ""
    set data [read $f]
         close $f
         return $data
}


# ---------------------------------------------------------------------
# Insert values of affinity to themes for each project
# ---------------------------------------------------------------------
proc assign_probs_projects {project_id theme_norm_values} {
	# Assign the normalized affinity values to the project in the database
	foreach tema $theme_norm_values {
		db_transaction {
			set t_p_a [lindex $tema 1]
			set t_id [lindex $tema 0]
			db_dml theme_add {
				insert into im_ef_projects(project_id,theme_id,theme_prob_assigned,theme_prob_calc)
				values(:project_id,:t_id,:t_p_a,0)		
			}
		} on_error {
			ad_return_error "Error adding probabilites for the project's themes" "Problems adding probabilites to 
			the theme [lindex $tema 0] of the project $project_id <br>
			Here the error message: <pre>$errmsg\n</pre>\n"
			return
		}		
	}
}


# ---------------------------------------------------------------------
# For each document processed we need an owner
# ---------------------------------------------------------------------
proc assign_owner_to_file { user_email the_file } {
	set user_name [lindex [split $user_email '@'] 0]
	set sql "select user_id from users where username like('$user_name%')"
	db_1row get_user_id $sql
		
	db_transaction {	
		db_dml doc_add {
			insert into im_ef_files(file_id,owner,path)
			values(im_ef_file_id_seq.nextval,:user_id,:the_file)		
		}
	} on_error {
		ad_return_error "Error expanding probabilites to project's documents" "Problems adding probabilites to 
		the document at $the_file <br>
		Here the error message: <pre>$errmsg\n</pre>\n"
		return	
	}
	
	db_1row get_id_value "select im_ef_file_id_seq.currval as f_id from dual"
	
	return $f_id
}


# ---------------------------------------------------------------------
# Get a list of words from a string
# ---------------------------------------------------------------------
proc extract_words { data } {
    set word_list [split [string tolower $data] " .,\n:?!;-\""]
    return $word_list
} 

# ---------------------------------------------------------------------
# Remove duplicates from the word list
# ---------------------------------------------------------------------
proc extract_distinct_words { word_list distinct_words_list} {
    global count_list

	foreach word $word_list {
	    	# The list is initialized
		if { [llength $distinct_words_list] > 0 } {
		    	set long [expr [llength $distinct_words_list] - 1]	    
		    	if { [string compare $word [lindex $distinct_words_list $long]] == 0} {
				# The word is already in the list, just update its number of occurrences
		       		set count_list [lreplace $count_list $long $long [expr [lindex $count_list $long] + 1]]
		    	} else {
				if { [regexp {[0-9]+} $word match] == 1 } {
					# Something containing numbers? Discard it 
				} else {
					# Add the word to the word list, and an entry to the count list 
			    		set distinct_words_list [lappend distinct_words_list $word]
			    		set count_list [lappend count_list 1]	
				}
		    	}
	       	} else {
	       	# The first insertion
	          lappend distinct_words_list $word
	          lappend count_list 1
	       }
    	}

    return $distinct_words_list
}


# ---------------------------------------------------------------------
# Assign the word's number of occurrences 
# ---------------------------------------------------------------------
proc assign_words_doc {file_id file_data} {
	global count_list
	global contingut
	#	Extract its distinct word list
	set word_list [lsort [extract_words $file_data]]
	set index_blank 0
	while { ([lindex $word_list $index_blank] == "") || ([lindex $word_list $index_blank] == " ") } {
	    incr index_blank
	}
	set word_list [lrange $word_list $index_blank [llength $word_list]]
	
	set distinct_words_list [list]
	#	Get a list of ocurrences of the distinct words in the document
	set distinct_words_list [extract_distinct_words $word_list $distinct_words_list]

	# insert each word into the postings table
	set counter 0 
	foreach paraula $distinct_words_list {
		set word_count [lindex $count_list $counter]
		db_transaction {
			db_dml theme_add {
				insert into im_ef_postings(keyword,file_id,occurrences)
				values(:paraula,:file_id,:word_count)		
			}
		} on_error {
			ad_return_error "Error adding number of occurrences to the postings table" "Problems adding number of occurrences 
			to the word '$paraula' <br>
			Here the error message: <pre>$errmsg\n</pre>\n"
			return
		}			
		incr counter
		append contingut "<BR>$counter - insert into im_ef_postings(keyword,file_id,occurrences) values($paraula,$file_id,$word_count)"
	}
			
}

# ---------------------------------------------------------------------
# Insert values of affinity to themes for each project
# ---------------------------------------------------------------------
proc assign_probs_doc {file_id theme_norm_values} {
	# Assign the normalized affinity values to the project in the database
	foreach tema $theme_norm_values {
		db_transaction {
			set t_p_a [lindex $tema 1]
			set t_id [lindex $tema 0]
			db_dml theme_add {
				insert into im_ef_documents(file_id,theme_id,theme_prob_assigned,theme_prob_calc)
				values(:file_id,:t_id,:t_p_a,0)		
			}
		} on_error {
			ad_return_error "Error adding probabilites for the document's themes" "Problems adding probabilites to 
			the theme [lindex $tema 0] of the document with file_id $file_id <br>
			Here the error message: <pre>$errmsg\n</pre>\n"
			return
		}		
	}
}

# ---------------------------------------------------------------------
# The process of trainning each file
# ---------------------------------------------------------------------
proc train_file {the_file theme_norm_values} {
	# SI NO EXISTEIX UNA ENTRADA A LA TAULA im_ef_files llavors.....  ################################################################################################### MISSING
		# Look for the file owner (first line in the file)
		set file_data [read_file $the_file]
		set lines [split $file_data "\n"]
		set user_email [lindex $lines 0]			
		set file_id [assign_owner_to_file $user_email $the_file]	
	# SI EXISTEIX, TINDREM EL SEU FILE_ID ################################################################################################### MISSING
	set foo [assign_probs_doc $file_id $theme_norm_values]
	set foo [assign_words_doc $file_id $file_data]
}



# ---------------------------------------------------------------------
# EXECUTING CODE
# ---------------------------------------------------------------------


# create a list with affinity values to each theme
set theme_ids [array names tema]
foreach id $theme_ids {
	set theme_temp [list]
	lappend theme_temp $id
	lappend theme_temp $tema($id)	
	lappend theme_values $theme_temp	
}

# Add all values to get the total amount of affinity
set total_values 0.0
foreach puntuacio $theme_values {
	set total_values [expr $total_values + [lindex $puntuacio 1]]	
}

# Normalize each value of affinity respect the total
set theme_norm_values [list]
foreach puntuacio $theme_values {
	set tmp_list [list]
	lappend tmp_list [lindex $puntuacio 0]
	if { $total_values != 0 } {
		lappend tmp_list [expr [lindex $puntuacio 1] / $total_values]
	} else {
		lappend tmp_list 0.0
	}
	
	lappend theme_norm_values $tmp_list
}

set tempo [assign_probs_projects $project_id $theme_norm_values]

# Create an entry if needed for each file
set files [im_filestorage_find_files $project_id]
foreach the_file $files {
	set var [train_file $the_file $theme_norm_values]
}

db_release_unused_handles 

doc_return  200 text/html $contingut