ad_page_contract {
    Administration page for intranet-search-oracle
    Calculates a user profile from its owned documents

    @param user_profile   The user we want to calculate its profile
    @author Marc Baiges Camprub (tm08707@salleurl.edu)
} {    
    user_profile:notnull
}

# ---------------------------------------------------------------------
# Defaults & Security
# ---------------------------------------------------------------------

set user_id [ad_maybe_redirect_for_registration]


# Initializations
set prob_accum 0.0
set doc_count 0
set theme_list [list]


# Delete current user profile to recalculate it
db_transaction {
    db_dml user_delete {
			delete from im_ef_users where
				user_id = :user_profile
    }

} on_error {
    ad_return_error "Error deleting a user's profile" "
The database rejected the deletion of the profile belonging to user \"$user_profile\". Here the error message: <pre>$errmsg\n</pre>\n"
    return
}


# Loop throught themes and get a value of affinity for each one
set sql "select distinct(theme_id) as tema from im_ef_themes"
db_foreach get_themes $sql {
	set sql2 "select theme_prob_assigned 
						from im_ef_documents 
							where theme_id=:tema and 
							file_id in (select file_id from im_ef_files where owner =:user_profile)
						"
	# Every user's document with probability about that theme scores
	db_foreach probs_theme $sql2 {
		incr doc_count
		set prob_accum [expr $prob_accum + $theme_prob_assigned]
	}                
	lappend theme_list [list $tema $prob_accum $doc_count]
	set prob_accum 0.0
	set doc_count 0
}

set contingut ""

# Insert into database the value calculated for each user's theme
foreach theme $theme_list {
	set tema [lindex $theme 0]
	set prob [expr [lindex $theme 1] / [lindex $theme 2]]
	db_transaction {	
		db_dml add {
			insert into im_ef_users(user_id,theme_id,theme_prob_assigned,theme_prob_calc)
			values(:user_profile,:tema,:prob,0)		
		}
	} on_error {
		ad_return_error "Error adding probabilites for users' themes" "Problems adding probabilites to 
		the user $user_profile on theme $tema <br>
		Here the error message: <pre>$errmsg\n</pre>\n"
		return
	}		
}

db_release_unused_handles 

ad_returnredirect ef_admin.tcl